@@ -30,12 +30,11 @@ import unittest
 from test import _common
 from test._common import item
 import beets.library
-import beets.mediafile
 import beets.dbcore.query
 from beets import util
 from beets import plugins
 from beets import config
-from beets.mediafile import MediaFile
+from mediafile import MediaFile, UnreadableFileError
 from beets.util import syspath, bytestring_path
 from test.helper import TestHelper
 import six
@@ -133,6 +132,21 @@ class GetSetTest(_common.TestCase):
     def test_invalid_field_raises_attributeerror(self):
         self.assertRaises(AttributeError, getattr, self.i, u'xyzzy')
 
+    def test_album_fallback(self):
+        # integration test of item-album fallback
+        lib = beets.library.Library(':memory:')
+        i = item(lib)
+        album = lib.add_album([i])
+        album['flex'] = u'foo'
+        album.store()
+
+        self.assertTrue('flex' in i)
+        self.assertFalse('flex' in i.keys(with_album=False))
+        self.assertEqual(i['flex'], u'foo')
+        self.assertEqual(i.get('flex'), u'foo')
+        self.assertEqual(i.get('flex', with_album=False), None)
+        self.assertEqual(i.get('flexx'), None)
+
 
 class DestinationTest(_common.TestCase):
     def setUp(self):
@@ -492,6 +506,24 @@ class DestinationTest(_common.TestCase):
         dest = self.i.destination()
         self.assertEqual(dest[-2:], b'XX')
 
+    def test_album_field_query(self):
+        self.lib.directory = b'one'
+        self.lib.path_formats = [(u'default', u'two'),
+                                 (u'flex:foo', u'three')]
+        album = self.lib.add_album([self.i])
+        self.assertEqual(self.i.destination(), np('one/two'))
+        album['flex'] = u'foo'
+        album.store()
+        self.assertEqual(self.i.destination(), np('one/three'))
+
+    def test_album_field_in_template(self):
+        self.lib.directory = b'one'
+        self.lib.path_formats = [(u'default', u'$flex/two')]
+        album = self.lib.add_album([self.i])
+        album['flex'] = u'foo'
+        album.store()
+        self.assertEqual(self.i.destination(), np('one/foo/two'))
+
 
 class ItemFormattedMappingTest(_common.LibTestCase):
     def test_formatted_item_value(self):
@@ -1169,7 +1201,7 @@ class ItemReadTest(unittest.TestCase):
         with self.assertRaises(beets.library.ReadError) as cm:
             item.read(unreadable)
         self.assertIsInstance(cm.exception.reason,
-                              beets.mediafile.UnreadableFileError)
+                              UnreadableFileError)
 
     def test_nonexistent_raise_read_error(self):
         item = beets.library.Item()
