@@ -29,10 +29,10 @@ import time
 import yaml
 import tempfile
 from contextlib import contextmanager
-import random
 
-from beets.util import confit, py3_path
+from beets.util import py3_path, bluelet
 from beetsplug import bpd
+import confuse
 
 
 # Mock GstPlayer so that the forked process doesn't attempt to import gi:
@@ -230,11 +230,6 @@ class MPCClient(object):
                 return line
 
 
-def start_beets(*args):
-    import beets.ui
-    beets.ui.main(list(args))
-
-
 def implements(commands, expectedFailure=False):  # noqa: N803
     def _test(self):
         with self.run_bpd() as client:
@@ -245,6 +240,29 @@ def implements(commands, expectedFailure=False):  # noqa: N803
     return unittest.expectedFailure(_test) if expectedFailure else _test
 
 
+bluelet_listener = bluelet.Listener
+
+
+@mock.patch("beets.util.bluelet.Listener")
+def start_server(args, assigned_port, listener_patch):
+    """Start the bpd server, writing the port to `assigned_port`.
+    """
+    def listener_wrap(host, port):
+        """Wrap `bluelet.Listener`, writing the port to `assigend_port`.
+        """
+        # `bluelet.Listener` has previously been saved to
+        # `bluelet_listener` as this function will replace it at its
+        # original location.
+        listener = bluelet_listener(host, port)
+        # read port assigned by OS
+        assigned_port.put_nowait(listener.sock.getsockname()[1])
+        return listener
+    listener_patch.side_effect = listener_wrap
+
+    import beets.ui
+    beets.ui.main(args)
+
+
 class BPDTestHelper(unittest.TestCase, TestHelper):
     def setUp(self):
         self.setup_beets(disk=True)
@@ -262,22 +280,18 @@ class BPDTestHelper(unittest.TestCase, TestHelper):
         self.unload_plugins()
 
     @contextmanager
-    def run_bpd(self, host='localhost', port=None, password=None,
-                do_hello=True, second_client=False):
+    def run_bpd(self, host='localhost', password=None, do_hello=True,
+                second_client=False):
         """ Runs BPD in another process, configured with the same library
         database as we created in the setUp method. Exposes a client that is
         connected to the server, and kills the server at the end.
         """
-        # Choose a port (randomly) to avoid conflicts between parallel
-        # tests.
-        if not port:
-            port = 9876 + random.randint(0, 10000)
-
         # Create a config file:
         config = {
                 'pluginpath': [py3_path(self.temp_dir)],
                 'plugins': 'bpd',
-                'bpd': {'host': host, 'port': port, 'control_port': port + 1},
+                # use port 0 to let the OS choose a free port
+                'bpd': {'host': host, 'port': 0, 'control_port': 0},
         }
         if password:
             config['bpd']['password'] = password
@@ -285,42 +299,43 @@ class BPDTestHelper(unittest.TestCase, TestHelper):
                 mode='wb', dir=py3_path(self.temp_dir), suffix='.yaml',
                 delete=False)
         config_file.write(
-                yaml.dump(config, Dumper=confit.Dumper, encoding='utf-8'))
+                yaml.dump(config, Dumper=confuse.Dumper, encoding='utf-8'))
         config_file.close()
 
         # Fork and launch BPD in the new process:
-        args = (
+        assigned_port = mp.Queue(2)  # 2 slots, `control_port` and `port`
+        server = mp.Process(target=start_server, args=([
             '--library', self.config['library'].as_filename(),
             '--directory', py3_path(self.libdir),
             '--config', py3_path(config_file.name),
             'bpd'
-        )
-        server = mp.Process(target=start_beets, args=args)
+        ], assigned_port))
         server.start()
 
-        # Wait until the socket is connected:
-        sock, sock2 = None, None
-        for _ in range(20):
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            if sock.connect_ex((host, port)) == 0:
-                break
-            else:
-                sock.close()
-                time.sleep(0.01)
-        else:
-            raise RuntimeError('Timed out waiting for the BPD server')
-
         try:
-            if second_client:
-                sock2 = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                sock2.connect((host, port))
-                yield MPCClient(sock, do_hello), MPCClient(sock2, do_hello)
-            else:
-                yield MPCClient(sock, do_hello)
+            assigned_port.get(timeout=1)  # skip control_port
+            port = assigned_port.get(timeout=0.5)  # read port
+
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            try:
+                sock.connect((host, port))
+
+                if second_client:
+                    sock2 = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                    try:
+                        sock2.connect((host, port))
+                        yield (
+                            MPCClient(sock, do_hello),
+                            MPCClient(sock2, do_hello),
+                        )
+                    finally:
+                        sock2.close()
+
+                else:
+                    yield MPCClient(sock, do_hello)
+            finally:
+                sock.close()
         finally:
-            sock.close()
-            if sock2:
-                sock2.close()
             server.terminate()
             server.join(timeout=0.2)
 
@@ -338,8 +353,9 @@ class BPDTestHelper(unittest.TestCase, TestHelper):
             previous_commands = response[0:pos]
             self._assert_ok(*previous_commands)
             response = response[pos]
-            self.assertEqual(pos, response.err_data[1])
         self.assertFalse(response.ok)
+        if pos is not None:
+            self.assertEqual(pos, response.err_data[1])
         if code is not None:
             self.assertEqual(code, response.err_data[0])
 
@@ -361,7 +377,7 @@ class BPDTestHelper(unittest.TestCase, TestHelper):
 class BPDTest(BPDTestHelper):
     def test_server_hello(self):
         with self.run_bpd(do_hello=False) as client:
-            self.assertEqual(client.readline(), b'OK MPD 0.14.0\n')
+            self.assertEqual(client.readline(), b'OK MPD 0.16.0\n')
 
     def test_unknown_cmd(self):
         with self.run_bpd() as client:
@@ -391,8 +407,31 @@ class BPDTest(BPDTestHelper):
 
 class BPDQueryTest(BPDTestHelper):
     test_implements_query = implements({
-            'clearerror', 'currentsong', 'stats',
-            })
+        'clearerror',
+    })
+
+    def test_cmd_currentsong(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1)
+            responses = client.send_commands(
+                    ('play',),
+                    ('currentsong',),
+                    ('stop',),
+                    ('currentsong',))
+        self._assert_ok(*responses)
+        self.assertEqual('1', responses[1].data['Id'])
+        self.assertNotIn('Id', responses[3].data)
+
+    def test_cmd_currentsong_tagtypes(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1)
+            responses = client.send_commands(
+                    ('play',),
+                    ('currentsong',))
+        self._assert_ok(*responses)
+        self.assertEqual(
+                BPDConnectionTest.TAGTYPES.union(BPDQueueTest.METADATA),
+                set(responses[1].data.keys()))
 
     def test_cmd_status(self):
         with self.run_bpd() as client:
@@ -409,10 +448,19 @@ class BPDQueryTest(BPDTestHelper):
         }
         self.assertEqual(fields_not_playing, set(responses[0].data.keys()))
         fields_playing = fields_not_playing | {
-            'song', 'songid', 'time', 'elapsed', 'bitrate', 'duration', 'audio'
+            'song', 'songid', 'time', 'elapsed', 'bitrate', 'duration',
+            'audio', 'nextsong', 'nextsongid'
         }
         self.assertEqual(fields_playing, set(responses[2].data.keys()))
 
+    def test_cmd_stats(self):
+        with self.run_bpd() as client:
+            response = client.send_command('stats')
+        self._assert_ok(response)
+        details = {'artists', 'albums', 'songs', 'uptime', 'db_playtime',
+                   'db_update', 'playtime'}
+        self.assertEqual(details, set(response.data.keys()))
+
     def test_cmd_idle(self):
         def _toggle(c):
             for _ in range(3):
@@ -448,6 +496,14 @@ class BPDQueryTest(BPDTestHelper):
             response = client.send_command('noidle')
         self._assert_ok(response)
 
+    def test_cmd_noidle_when_not_idle(self):
+        with self.run_bpd() as client:
+            # Manually send a command without reading a response.
+            request = client.serialise_command('noidle')
+            client.sock.sendall(request)
+            response = client.send_command('notacommand')
+        self._assert_failed(response, bpd.ERROR_UNKNOWN)
+
 
 class BPDPlaybackTest(BPDTestHelper):
     test_implements_playback = implements({
@@ -614,8 +670,13 @@ class BPDPlaybackTest(BPDTestHelper):
 
     def test_cmd_volume(self):
         with self.run_bpd() as client:
-            response = client.send_command('volume', '10')
-        self._assert_failed(response, bpd.ERROR_SYSTEM)
+            responses = client.send_commands(
+                    ('setvol', '10'),
+                    ('volume', '5'),
+                    ('volume', '-2'),
+                    ('status',))
+        self._assert_ok(*responses)
+        self.assertEqual('13', responses[3].data['volume'])
 
     def test_cmd_replay_gain(self):
         with self.run_bpd() as client:
@@ -629,9 +690,8 @@ class BPDPlaybackTest(BPDTestHelper):
 
 class BPDControlTest(BPDTestHelper):
     test_implements_control = implements({
-            'pause', 'playid', 'seek',
-            'seekid', 'seekcur', 'stop',
-            }, expectedFailure=True)
+        'seek', 'seekid', 'seekcur',
+    }, expectedFailure=True)
 
     def test_cmd_play(self):
         with self.run_bpd() as client:
@@ -647,6 +707,45 @@ class BPDControlTest(BPDTestHelper):
         self.assertEqual('play', responses[2].data['state'])
         self.assertEqual('2', responses[4].data['Id'])
 
+    def test_cmd_playid(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1, self.item2)
+            responses = client.send_commands(
+                    ('playid', '2'),
+                    ('currentsong',),
+                    ('clear',))
+            self._bpd_add(client, self.item2, self.item1)
+            responses.extend(client.send_commands(
+                    ('playid', '2'),
+                    ('currentsong',)))
+        self._assert_ok(*responses)
+        self.assertEqual('2', responses[1].data['Id'])
+        self.assertEqual('2', responses[4].data['Id'])
+
+    def test_cmd_pause(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1)
+            responses = client.send_commands(
+                    ('play',),
+                    ('pause',),
+                    ('status',),
+                    ('currentsong',))
+        self._assert_ok(*responses)
+        self.assertEqual('pause', responses[2].data['state'])
+        self.assertEqual('1', responses[3].data['Id'])
+
+    def test_cmd_stop(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1)
+            responses = client.send_commands(
+                    ('play',),
+                    ('stop',),
+                    ('status',),
+                    ('currentsong',))
+        self._assert_ok(*responses)
+        self.assertEqual('stop', responses[2].data['state'])
+        self.assertNotIn('Id', responses[3].data)
+
     def test_cmd_next(self):
         with self.run_bpd() as client:
             self._bpd_add(client, self.item1, self.item2)
@@ -683,24 +782,48 @@ class BPDControlTest(BPDTestHelper):
 class BPDQueueTest(BPDTestHelper):
     test_implements_queue = implements({
             'addid', 'clear', 'delete', 'deleteid', 'move',
-            'moveid', 'playlist', 'playlistfind', 'playlistid',
+            'moveid', 'playlist', 'playlistfind',
             'playlistsearch', 'plchanges',
             'plchangesposid', 'prio', 'prioid', 'rangeid', 'shuffle',
             'swap', 'swapid', 'addtagid', 'cleartagid',
             }, expectedFailure=True)
 
+    METADATA = {'Pos', 'Time', 'Id', 'file', 'duration'}
+
     def test_cmd_add(self):
         with self.run_bpd() as client:
             self._bpd_add(client, self.item1)
 
     def test_cmd_playlistinfo(self):
         with self.run_bpd() as client:
-            self._bpd_add(client, self.item1)
+            self._bpd_add(client, self.item1, self.item2)
             responses = client.send_commands(
                     ('playlistinfo',),
                     ('playlistinfo', '0'),
+                    ('playlistinfo', '0:2'),
                     ('playlistinfo', '200'))
-        self._assert_failed(responses, bpd.ERROR_ARG, pos=2)
+        self._assert_failed(responses, bpd.ERROR_ARG, pos=3)
+        self.assertEqual('1', responses[1].data['Id'])
+        self.assertEqual(['1', '2'], responses[2].data['Id'])
+
+    def test_cmd_playlistinfo_tagtypes(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1)
+            response = client.send_command('playlistinfo', '0')
+        self._assert_ok(response)
+        self.assertEqual(
+                BPDConnectionTest.TAGTYPES.union(BPDQueueTest.METADATA),
+                set(response.data.keys()))
+
+    def test_cmd_playlistid(self):
+        with self.run_bpd() as client:
+            self._bpd_add(client, self.item1, self.item2)
+            responses = client.send_commands(
+                    ('playlistid', '2'),
+                    ('playlistid',))
+        self._assert_ok(*responses)
+        self.assertEqual('Track Two Title', responses[0].data['Title'])
+        self.assertEqual(['1', '2'], responses[1].data['Track'])
 
 
 class BPDPlaylistsTest(BPDTestHelper):
@@ -830,8 +953,24 @@ class BPDStickerTest(BPDTestHelper):
 
 class BPDConnectionTest(BPDTestHelper):
     test_implements_connection = implements({
-            'close', 'kill', 'tagtypes',
-            })
+        'close', 'kill',
+    })
+
+    ALL_MPD_TAGTYPES = {
+        'Artist', 'ArtistSort', 'Album', 'AlbumSort', 'AlbumArtist',
+        'AlbumArtistSort', 'Title', 'Track', 'Name', 'Genre', 'Date',
+        'Composer', 'Performer', 'Comment', 'Disc', 'Label',
+        'OriginalDate', 'MUSICBRAINZ_ARTISTID', 'MUSICBRAINZ_ALBUMID',
+        'MUSICBRAINZ_ALBUMARTISTID', 'MUSICBRAINZ_TRACKID',
+        'MUSICBRAINZ_RELEASETRACKID', 'MUSICBRAINZ_WORKID',
+    }
+    UNSUPPORTED_TAGTYPES = {
+        'MUSICBRAINZ_WORKID',  # not tracked by beets
+        'Performer',           # not tracked by beets
+        'AlbumSort',           # not tracked by beets
+        'Name',                # junk field for internet radio
+    }
+    TAGTYPES = ALL_MPD_TAGTYPES.difference(UNSUPPORTED_TAGTYPES)
 
     def test_cmd_password(self):
         with self.run_bpd(password='abc123') as client:
@@ -851,19 +990,13 @@ class BPDConnectionTest(BPDTestHelper):
             response = client.send_command('ping')
         self._assert_ok(response)
 
-    @unittest.skip
     def test_cmd_tagtypes(self):
         with self.run_bpd() as client:
             response = client.send_command('tagtypes')
         self._assert_ok(response)
-        self.assertEqual({
-            'Artist', 'ArtistSort', 'Album', 'AlbumSort', 'AlbumArtist',
-            'AlbumArtistSort', 'Title', 'Track', 'Name', 'Genre', 'Date',
-            'Composer', 'Performer', 'Comment', 'Disc', 'Label',
-            'OriginalDate', 'MUSICBRAINZ_ARTISTID', 'MUSICBRAINZ_ALBUMID',
-            'MUSICBRAINZ_ALBUMARTISTID', 'MUSICBRAINZ_TRACKID',
-            'MUSICBRAINZ_RELEASETRACKID', 'MUSICBRAINZ_WORKID',
-            }, set(response.data['tag']))
+        self.assertEqual(
+                self.TAGTYPES,
+                set(response.data['tagtype']))
 
     @unittest.skip
     def test_tagtypes_mask(self):
